# -*- coding: utf-8 -*-

"""
   Pinch.Models.Webhook
 
   This file was automatically generated for Pinch by APIMATIC v2.0 ( https://apimatic.io ) on 05/13/2016
"""
from Pinch.Models.BaseModel import BaseModel

class Webhook(BaseModel):

    """Implementation of the 'Webhook' model.

    TODO: type model description here.

    Attributes:
        id (int): TODO: type description here.
        url (string): TODO: type description here.
        webhook_type (int): TODO: type description here.

    """

    def __init__(self, 
                 id = None,
                 url = None,
                 webhook_type = None):
        """Constructor for the Webhook class"""
        
        # Initialize members of the class
        self.id = id
        self.url = url
        self.webhook_type = webhook_type

        # Create a mapping from Model property names to API property names
        self.names = {
            "id": "id",
            "url": "url",
            "webhook_type": "webhook_type",
        }

    @classmethod
    def from_dictionary(cls, 
                        dictionary):
        """Creates an instance of this model from a dictionary
       
        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.
            
        Returns:
            object: An instance of this structure class.
            
        """

        if dictionary == None:
            return None
        else:	
            # Extract variables from the dictionary
            id = dictionary.get("id")
            url = dictionary.get("url")
            webhook_type = dictionary.get("webhook_type")
            # Return an object of this model
            return cls(id,
                       url,
                       webhook_type)



# -*- coding: utf-8 -*-

'''
 Pinch

 This file was automatically generated for Pinch by APIMATIC v2.0 ( https://apimatic.io ) on 05/13/2016
'''
from Configuration import *

class CustomAuthUtility:

    @staticmethod
    def appendCustomAuthParams(http_request):
        ''' Add custom authentication to the request.

        Args:
            http_request (HttpRequest object): The HttpRequest object with fields you can modify.
        '''

        # TODO: Add your custom authentication here
        # The following properties are available to use
        #     Configuration.x_api_token
        #     Configuration.x_api_email
        # 
        # example: 
        # Add a header through: http_request.headers["key"] = "value"



"""
    Pinch

    This file was automatically generated for Pinch by APIMATIC BETA v2.0 on 05/13/2016
"""
from Pinch.Models import *
from Pinch.Controllers import *
from Pinch.Decorators import *
from Pinch.APIException import *

class PinchClient(object):

    @lazy_property
    def webhook_type_controller(self):
        return WebhookTypeController()

    @lazy_property
    def webhook_controller(self):
        return WebhookController()

    @lazy_property
    def ticket_controller(self):
        return TicketController()






# -*- coding: utf-8 -*-

"""
   Pinch.APIException

   This file was automatically generated for Pinch by APIMATIC v2.0 ( https://apimatic.io ) on 05/13/2016
"""

class lazy_property(object):

    """A decorator class for lazy instantiation."""

    def __init__(self,fget):
        self.fget = fget
        self.func_name = fget.__name__

    def __get__(self,obj,cls):
        if obj is None:
            return None
        value = self.fget(obj)
        setattr(obj,self.func_name,value)
        return value


# -*- coding: utf-8 -*-

"""
   Pinch.Controllers.WebhookTypeController

   This file was automatically generated for Pinch by APIMATIC BETA v2.0 on 05/13/2016
"""
from Pinch.APIHelper import APIHelper
from Pinch.APIException import APIException
from Pinch.Configuration import Configuration
from Pinch.Http.HttpRequest import HttpRequest
from Pinch.Http.HttpResponse import HttpResponse
from Pinch.Http.RequestsClient import RequestsClient
from Pinch.Controllers.BaseController import BaseController
from Pinch.CustomAuthUtility import CustomAuthUtility

from Pinch.Models.WebhookType import WebhookType


class WebhookTypeController(BaseController):

    """A Controller to access Endpoints in the Pinch API."""

    def __init__(self, http_client = None):
        """Constructor which allows a different HTTP client for this controller."""
        BaseController.__init__(self, http_client)

    def list(self):
        """Does a GET request to /webhook_types.

        List webhook types

        Returns:
            list of WebhookType: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # The base uri for api requests
        _query_builder = Configuration.BASE_URI
 
        # Prepare query string for API call
        _query_builder += "/webhook_types"
        
        # Validate and preprocess url
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            "user-agent": "APIMATIC 2.0",
            "accept": "application/json",
            "X-API-TOKEN": Configuration.x_api_token,
            "X-API-EMAIL": Configuration.x_api_email
        }

        # Prepare the API call.
        _http_request = self.http_client.get(_query_url, headers=_headers)

        #append custom auth authorization
        CustomAuthUtility.appendCustomAuthParams(_http_request)

        # Invoke the API call  to fetch the response.
        _response = self.http_client.execute_as_string(_http_request)

        # Global error handling using HTTP status codes.
        self.validate_response(_response)    

        # Try to convert response to JSON
        try:
            _response.raw_body = APIHelper.json_deserialize(_response.raw_body)
        except:
            pass
        
        # Try to cast response to list of desired type
        if isinstance(_response.raw_body, list):
            # Response is already in a list, return the list of objects 
            value = list()
            for item in _response.raw_body:
                try:
                    value.append(WebhookType.from_dictionary(item))
                except Exception:
                    raise APIException("Invalid JSON returned.", _response.status_code, _response.raw_body)
                    
            return value



# -*- coding: utf-8 -*-

"""
   Pinch.Controllers.WebhookController

   This file was automatically generated for Pinch by APIMATIC BETA v2.0 on 05/13/2016
"""
from Pinch.APIHelper import APIHelper
from Pinch.APIException import APIException
from Pinch.Configuration import Configuration
from Pinch.Http.HttpRequest import HttpRequest
from Pinch.Http.HttpResponse import HttpResponse
from Pinch.Http.RequestsClient import RequestsClient
from Pinch.Controllers.BaseController import BaseController
from Pinch.CustomAuthUtility import CustomAuthUtility

from Pinch.Models.Webhook import Webhook


class WebhookController(BaseController):

    """A Controller to access Endpoints in the Pinch API."""

    def __init__(self, http_client = None):
        """Constructor which allows a different HTTP client for this controller."""
        BaseController.__init__(self, http_client)

    def list(self):
        """Does a GET request to /webhooks.

        List the webhooks of the current user

        Returns:
            list of Webhook: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # The base uri for api requests
        _query_builder = Configuration.BASE_URI
 
        # Prepare query string for API call
        _query_builder += "/webhooks"
        
        # Validate and preprocess url
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            "user-agent": "APIMATIC 2.0",
            "accept": "application/json",
            "X-API-TOKEN": Configuration.x_api_token,
            "X-API-EMAIL": Configuration.x_api_email
        }

        # Prepare the API call.
        _http_request = self.http_client.get(_query_url, headers=_headers)

        #append custom auth authorization
        CustomAuthUtility.appendCustomAuthParams(_http_request)

        # Invoke the API call  to fetch the response.
        _response = self.http_client.execute_as_string(_http_request)

        # Global error handling using HTTP status codes.
        self.validate_response(_response)    

        # Try to convert response to JSON
        try:
            _response.raw_body = APIHelper.json_deserialize(_response.raw_body)
        except:
            pass
        
        # Try to cast response to list of desired type
        if isinstance(_response.raw_body, list):
            # Response is already in a list, return the list of objects 
            value = list()
            for item in _response.raw_body:
                try:
                    value.append(Webhook.from_dictionary(item))
                except Exception:
                    raise APIException("Invalid JSON returned.", _response.status_code, _response.raw_body)
                    
            return value

    def create(self,
                webhook):
        """Does a POST request to /webhooks.

        TODO: type endpoint description here.

        Args:
            webhook (Webhook): TODO: type description here. Example: 

        Returns:
            Webhook: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Validate required parameters
        if webhook == None:
            raise ValueError("Required parameter 'webhook' cannot be None.")

        # The base uri for api requests
        _query_builder = Configuration.BASE_URI
 
        # Prepare query string for API call
        _query_builder += "/webhooks"
        
        # Validate and preprocess url
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            "user-agent": "APIMATIC 2.0",
            "accept": "application/json",
            "content-type": "application/json; charset=utf-8",
            "X-API-TOKEN": Configuration.x_api_token,
            "X-API-EMAIL": Configuration.x_api_email
        }

        # Prepare the API call.
        _http_request = self.http_client.post(_query_url, headers=_headers, parameters=APIHelper.json_serialize(webhook))

        #append custom auth authorization
        CustomAuthUtility.appendCustomAuthParams(_http_request)

        # Invoke the API call  to fetch the response.
        _response = self.http_client.execute_as_string(_http_request)

        # Global error handling using HTTP status codes.
        self.validate_response(_response)    

        # Try to convert response to JSON
        try:
            _response.raw_body = APIHelper.json_deserialize(_response.raw_body)
        except:
            pass

        # Try to cast response to desired type
        if isinstance(_response.raw_body, dict):
            # Response is already in a dictionary, return the object 
            try:
                return Webhook.from_dictionary(_response.raw_body)
            except Exception:
                raise APIException("Invalid JSON returned.", _response.status_code, _response.raw_body)

    def update(self,
                webhook_id,
                webhook = None):
        """Does a PUT request to /webhooks/{webhook_id}.

        TODO: type endpoint description here.

        Args:
            webhook_id (int): TODO: type description here. Example: 
            webhook (Webhook, optional): TODO: type description here. Example:
                
        Returns:
            Webhook: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Validate required parameters
        if webhook_id == None:
            raise ValueError("Required parameter 'webhook_id' cannot be None.")

        # The base uri for api requests
        _query_builder = Configuration.BASE_URI
 
        # Prepare query string for API call
        _query_builder += "/webhooks/{webhook_id}"

        # Process optional template parameters
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            "webhook_id": webhook_id
        })
        
        # Validate and preprocess url
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            "user-agent": "APIMATIC 2.0",
            "accept": "application/json",
            "content-type": "application/json; charset=utf-8",
            "X-API-TOKEN": Configuration.x_api_token,
            "X-API-EMAIL": Configuration.x_api_email
        }

        # Prepare the API call.
        _http_request = self.http_client.put(_query_url, headers=_headers, parameters=APIHelper.json_serialize(webhook))

        #append custom auth authorization
        CustomAuthUtility.appendCustomAuthParams(_http_request)

        # Invoke the API call  to fetch the response.
        _response = self.http_client.execute_as_string(_http_request)

        # Global error handling using HTTP status codes.
        self.validate_response(_response)    

        # Try to convert response to JSON
        try:
            _response.raw_body = APIHelper.json_deserialize(_response.raw_body)
        except:
            pass

        # Try to cast response to desired type
        if isinstance(_response.raw_body, dict):
            # Response is already in a dictionary, return the object 
            try:
                return Webhook.from_dictionary(_response.raw_body)
            except Exception:
                raise APIException("Invalid JSON returned.", _response.status_code, _response.raw_body)

    def destroy(self,
                webhook_id):
        """Does a DELETE request to /webhooks/{webhook_id}.

        TODO: type endpoint description here.

        Args:
            webhook_id (int): TODO: type description here. Example: 

        Returns:
            string: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Validate required parameters
        if webhook_id == None:
            raise ValueError("Required parameter 'webhook_id' cannot be None.")

        # The base uri for api requests
        _query_builder = Configuration.BASE_URI
 
        # Prepare query string for API call
        _query_builder += "/webhooks/{webhook_id}"

        # Process optional template parameters
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            "webhook_id": webhook_id
        })
        
        # Validate and preprocess url
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            "user-agent": "APIMATIC 2.0",
            "X-API-TOKEN": Configuration.x_api_token,
            "X-API-EMAIL": Configuration.x_api_email
        }

        # Prepare the API call.
        _http_request = self.http_client.delete(_query_url, headers=_headers)

        #append custom auth authorization
        CustomAuthUtility.appendCustomAuthParams(_http_request)

        # Invoke the API call  to fetch the response.
        _response = self.http_client.execute_as_string(_http_request)

        # Global error handling using HTTP status codes.
        self.validate_response(_response)    

        # Return appropriate type
        return _response.raw_body



    def get(self,
                id):
        """Does a GET request to /webhooks/{id}.

        Get a specific webhook by its id

        Args:
            id (string): TODO: type description here. Example: 

        Returns:
            Webhook: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Validate required parameters
        if id == None:
            raise ValueError("Required parameter 'id' cannot be None.")

        # The base uri for api requests
        _query_builder = Configuration.BASE_URI
 
        # Prepare query string for API call
        _query_builder += "/webhooks/{id}"

        # Process optional template parameters
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            "id": id
        })
        
        # Validate and preprocess url
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            "user-agent": "APIMATIC 2.0",
            "accept": "application/json",
            "X-API-TOKEN": Configuration.x_api_token,
            "X-API-EMAIL": Configuration.x_api_email
        }

        # Prepare the API call.
        _http_request = self.http_client.get(_query_url, headers=_headers)

        #append custom auth authorization
        CustomAuthUtility.appendCustomAuthParams(_http_request)

        # Invoke the API call  to fetch the response.
        _response = self.http_client.execute_as_string(_http_request)

        # Global error handling using HTTP status codes.
        self.validate_response(_response)    

        # Try to convert response to JSON
        try:
            _response.raw_body = APIHelper.json_deserialize(_response.raw_body)
        except:
            pass

        # Try to cast response to desired type
        if isinstance(_response.raw_body, dict):
            # Response is already in a dictionary, return the object 
            try:
                return Webhook.from_dictionary(_response.raw_body)
            except Exception:
                raise APIException("Invalid JSON returned.", _response.status_code, _response.raw_body)



from Pinch.Models.WebhookType import *
from Pinch.Models.Webhook import *
from Pinch.Models.Ticket import *
from Pinch.Models.Person import *
from Pinch.Models.Building import *
from Pinch.Models.Unit import *
from Pinch.Models.Document import *



# -*- coding: utf-8 -*-

"""
   Pinch.Models.Unit
 
   This file was automatically generated for Pinch by APIMATIC v2.0 ( https://apimatic.io ) on 05/13/2016
"""
from Pinch.Models.BaseModel import BaseModel

class Unit(BaseModel):

    """Implementation of the 'Unit' model.

    TODO: type model description here.

    Attributes:
        reference (string): TODO: type description here.
        tenant_name (string): TODO: type description here.
        floor_number (string): TODO: type description here.
        kind (string): TODO: type description here.
        french_floor_number (string): TODO: type description here.

    """

    def __init__(self, 
                 reference = None,
                 tenant_name = None,
                 floor_number = None,
                 kind = None,
                 french_floor_number = None):
        """Constructor for the Unit class"""
        
        # Initialize members of the class
        self.reference = reference
        self.tenant_name = tenant_name
        self.floor_number = floor_number
        self.kind = kind
        self.french_floor_number = french_floor_number

        # Create a mapping from Model property names to API property names
        self.names = {
            "reference": "reference",
            "tenant_name": "tenant_name",
            "floor_number": "floor_number",
            "kind": "kind",
            "french_floor_number": "french_floor_number",
        }

    @classmethod
    def from_dictionary(cls, 
                        dictionary):
        """Creates an instance of this model from a dictionary
       
        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.
            
        Returns:
            object: An instance of this structure class.
            
        """

        if dictionary == None:
            return None
        else:	
            # Extract variables from the dictionary
            reference = dictionary.get("reference")
            tenant_name = dictionary.get("tenant_name")
            floor_number = dictionary.get("floor_number")
            kind = dictionary.get("kind")
            french_floor_number = dictionary.get("french_floor_number")
            # Return an object of this model
            return cls(reference,
                       tenant_name,
                       floor_number,
                       kind,
                       french_floor_number)



from Pinch.APIException import *
from Pinch.APIHelper import *
from Pinch.Configuration import *
from Pinch.Models import *
from Pinch.Controllers import *
from Pinch.Decorators import *
from Pinch.PinchClient import *



# -*- coding: utf-8 -*-

"""
   Pinch.Models.Document
 
   This file was automatically generated for Pinch by APIMATIC v2.0 ( https://apimatic.io ) on 05/13/2016
"""
from Pinch.Models.BaseModel import BaseModel

class Document(BaseModel):

    """Implementation of the 'Document' model.

    TODO: type model description here.

    Attributes:
        url (string): Where to retrieve the document

    """

    def __init__(self, 
                 url = None):
        """Constructor for the Document class"""
        
        # Initialize members of the class
        self.url = url

        # Create a mapping from Model property names to API property names
        self.names = {
            "url": "url",
        }

    @classmethod
    def from_dictionary(cls, 
                        dictionary):
        """Creates an instance of this model from a dictionary
       
        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.
            
        Returns:
            object: An instance of this structure class.
            
        """

        if dictionary == None:
            return None
        else:	
            # Extract variables from the dictionary
            url = dictionary.get("url")
            # Return an object of this model
            return cls(url)



# -*- coding: utf-8 -*-

"""
   Pinch

   This file was automatically generated for Pinch by APIMATIC BETA v2.0 on 05/13/2016
"""

from Pinch.Http.RequestsClient import *
from Pinch.APIException import APIException

class BaseController(object):

    """All controllers inherit from this base class. It manages shared HTTP clients and global API errors."""

    http_client = RequestsClient()

    def __init__(self, client):
        if client != None:
            self.http_client = client

    def validate_response(self, response):
        if response.status_code == 401:
            raise APIException("Your API key is incorrect", response.status_code, response.raw_body)
        elif response.status_code == 400:
            raise APIException("There is an error in the parameters you send", response.status_code, response.raw_body)
        elif response.status_code == 404:
            raise APIException("Cannot find the resource specified", response.status_code, response.raw_body)
        elif (response.status_code < 200) or (response.status_code > 206): #[200,206] = HTTP OK
            raise APIException("HTTP Response Not OK", response.status_code, response.raw_body)



# -*- coding: utf-8 -*-

"""
   Pinch.Configuration

   This file was automatically generated for Pinch by APIMATIC v2.0 ( https://apimatic.io ) on 05/13/2016
"""

class Configuration(object):
    # The base Uri for API calls
    BASE_URI = "https://api-company.inchbase.com/api/v1"

    # Your api key
    # TODO: Replace the x_api_token with an appropriate value
    x_api_token = "TODO: Replace"

    # Your registered email
    # TODO: Replace the x_api_email with an appropriate value
    x_api_email = "TODO: Replace"


